@@ -27,12 +27,19 @@
 #define Rinherit_patt(typ) {prf_desc = Rinherit(typ); _}
 #endif
 
+#if OCAML_VERSION < (4, 11, 0)
+#define Pconst_string_patt(s, loc) Pconst_string (s, loc)
+#else
+#define Pconst_string_patt(s, loc) Pconst_string (s, loc, _)
+#endif
+
 open Longident
 open Location
 open Asttypes
 open Parsetree
 open Ast_helper
 open Ast_convenience
+open Ppx_deriving_runtime
 
 #if OCAML_VERSION >= (4, 05, 0)
 type tyvar = string Location.loc
@@ -142,6 +149,39 @@ let create =
 let string_of_core_type typ =
   Format.asprintf "%a" Pprintast.core_type { typ with ptyp_attributes = [] }
 
+type constant =
+  #if OCAML_VERSION >= (4, 03, 0)
+    Parsetree.constant
+  #else
+    Asttypes.constant
+  #endif
+
+let string_of_constant_opt (constant : constant) : string option =
+  match constant with
+  | Pconst_string_patt(s, _) -> Some s
+  | _ -> None
+
+let string_of_expression_opt (e : Parsetree.expression) : string option =
+  match e with
+  | { pexp_desc = Pexp_constant constant } ->
+      string_of_constant_opt constant
+  | _ -> None
+
+#if OCAML_VERSION >= (4, 03, 0)
+  module Const = Ast_helper.Const
+#else
+  module Const = struct
+    let integer ?suffix:_ i = Const_int (int_of_string i)
+    let int ?suffix:_ i = Const_int i
+    let int32 ?suffix:_ i = Const_int (Int32.to_int i)
+    let int64 ?suffix:_ i = Const_int (Int64.to_int i)
+    let nativeint ?suffix:_ i = Const_int (Nativeint.to_int i)
+    let float ?suffix:_ f = Const_float f
+    let char c = Const_char c
+    let string ?quotation_delimiter s = Const_string (s, quotation_delimiter)
+  end
+#endif
+
 module Arg = struct
   type 'a conv = expression -> ('a, string) Result.result
 
@@ -164,9 +204,7 @@ module Arg = struct
     | _ -> Error "boolean"
 
   let string expr =
-    match expr with
-    | { pexp_desc = Pexp_constant (Pconst_string (n, None)) } -> Ok n
-    | _ -> Error "string"
+    Option.to_result ~none:"string" (string_of_expression_opt expr)
 
   let char = function
     | { pexp_desc = Pexp_constant (Pconst_char c) } -> Ok c
@@ -400,7 +438,7 @@ let free_vars_in_core_type typ =
   in
   let uniq lst =
     let module StringSet = Set.Make(String) in
-    let add name (names, txts) =
+    let add (rev_names, txts) name =
       let txt =
 #if OCAML_VERSION >= (4, 05, 0)
         name.txt
@@ -409,9 +447,9 @@ let free_vars_in_core_type typ =
 #endif
       in
       if StringSet.mem txt txts
-      then (names, txts)
-      else (name :: names, StringSet.add txt txts)
-    in fst (List.fold_right add lst ([], StringSet.empty))
+      then (rev_names, txts)
+      else (name :: rev_names, StringSet.add txt txts)
+    in List.rev (fst (List.fold_left add ([], StringSet.empty) lst))
   in free_in typ |> uniq
 
 let var_name_of_int i =
@@ -623,7 +661,14 @@ let mapper =
   let module_nesting = ref [] in
   let with_module name f =
     let old_nesting = !module_nesting in
+#if OCAML_VERSION >= (4, 10, 0)
+    begin match name with
+    | Some name -> module_nesting := !module_nesting @ [name]
+    | None -> ()
+    end;
+#else
     module_nesting := !module_nesting @ [name];
+#endif
     let result = f () in
     module_nesting := old_nesting;
     result
